#!/bin/bash

#####
#	this program interactively asks for the input to run
#	DOCWBREG or IDOREG
# THIS IS CURRENTLY SET UP ASSUMING THAT THE DOCWBREG IN THE CURRENT
# DIRECTORY IS FOR NORTH AMERICA
#
# YEAR MO DY HR MN SC MSC  LAT     LON    DEP  MAG REG  NEIC FELTID STATE/COUNTRY
#####

DIALOG=${DIALOG=dialog}

#####
#	initialize
#####
YEAR=""
MONTH=""
DAY=""
HOUR=""
MINUTE=""
SECOND=""
MSEC="000"
LATITUDE=""
LONGITUDE=""
DEPTH=""
MAG=""
REGION="RFTN"
STATE=""
EVID="NONE"
FEVID="NONE"
SCRIPT="DOIRISRFTN"
DECON="DECONNONE"

RETURNVALUE=""
tempfile=`tempfile 2>/dev/null` || tempfile=/tmp/test$$
trap "rm -f $tempfile" 0 1 2 5 15

function testrange {
#####
#	VALUE MIN MAX TYPE FORMAT
#	where TYPE IS FLOAT or INT
#####
VALUE=$1
MIN=$2
MAX=$3
TYPE=$4
FORMAT=$5
}

isLeapYear="NO"
MonthDay="31"

function isleap {
#####
#	isleap YEAR
#       sets variable isLeapYear to "NO" or to "YES"
#####
year=$1    # init input variable
isLeapYear="NO" # init return variable

# rule 1
if (( year%4==0 )); then
    isLeapYear="YES"
    # rule 2 (exception to rule 1)
    if (( year%100==0 )); then
        isLeapYear="NO"
        # rule 3 (exception to rule 2)
        if (( year%400==0 )); then
            isLeapYear="YES"
        fi
    fi
fi
}

function MonthDays {
#####
#	MonthDays Month Year
#		sets variable MonthDay
#####
tMonth=$1
Year=$2
case ${Month} in
	1|01) MonthDay="31" ; tMonth=1 ;;
	2|02) 
		isleap ${Year}
                if [ "${isLeapYear}" = "YES" ]
		then
			MonthDay="29"
		else
			MonthDay="28"
		fi
                tMonth=2
		;;
	3|03) MonthDay="31" ; tMonth=3 ;;
	4|04) MonthDay="30" ; tMonth=4 ;;
	5|05) MonthDay="31" ; tMonth=5 ;;
	6|06) MonthDay="30" ; tMonth=6 ;;
	7|07) MonthDay="31" ; tMonth=7 ;;
	8|08) MonthDay="31" ; tMonth=8 ;;
	9|09) MonthDay="30" ; tMonth=9 ;;
	10) MonthDay="31" ; tMonth=10 ;;
	11) MonthDay="30" ; tMonth=11 ;;
	12) MonthDay="31" ; tMonth=12 ;;
esac
}

function CheckDate {
#####
#	CheckDate YEAR MONTH DAY
#
#       return BAD is not correct
#       return INCOMPLETE IS NOT ALL FIELDS DEFINED
#       return OK if correct
YR="$1"
case $2 in
	01) MO="1" ;;
	02) MO="2" ;;
	03) MO="3" ;;
	04) MO="4" ;;
	05) MO="5" ;;
	06) MO="6" ;;
	07) MO="7" ;;
	08) MO="8" ;;
	09) MO="9" ;;
	*)  MO="$2" ;;
esac
case $3 in
	01) DY="1" ;;
	02) DY="2" ;;
	03) DY="3" ;;
	04) DY="4" ;;
	05) DY="5" ;;
	06) DY="6" ;;
	07) DY="7" ;;
	08) DY="8" ;;
	09) DY="9" ;;
	*)  DY="$3" ;;
esac

CHECKDATE="OK"
if [ "${YR}" != "" ]
then
	if [ "${MO}" != "" ]
	then
		if [ "${DY}" != "" ]
		then
                	MonthDays ${MO} ${YR}
         		CHECKDATE=`echo ${DY} ${MonthDay} | awk '{ANS="OK";if($1 > $2)ANS="BAD";print ANS}' `
		fi
	fi
fi
#####
#	now output the error message
#####
if [ ${CHECKDATE} != "OK" ]
then
	left=3
	while test $left != 0
		do
		$DIALOG --sleep 1 \
        		--title "Date Inconsistency" \
        		--infobox "The input value YEAR MONTH DAY \n                ${YR}  ${MO}   ${DY}\nis not consistent. This month has only ${MonthDay} days\n You have $left $unit to read this..." 10 52
		left=`expr $left - 1`
		test $left = 1 && unit="second"
	done
fi
}

function getreturnvalue {
#####
#	getreturnvalue CURRENT MIN MAX FORMAT TYPE TEXT TITLE
#		places the answer in RETURNVALUE
#####
CURRENT="$1"
MIN="$2"
MAX="$3"
FORMAT="$4"
TYPE="$5"
TEXT="$6"
TITLE="$7"
DONE="NO"

#####
#	parameters for infobox
#####
left=3
unit="seconds"

while [ ${DONE} = "NO" ]
do
$DIALOG --title "${TITLE}" --clear \
     --inputbox "${TEXT}" 15 51 "${CURRENT}" 2> $tempfile
retval=$?
case $retval in
  0)
	VAL=`cat $tempfile`
	 VAL=`printf  "${FORMAT}" "${VAL}"`
	DONE="YES"
#####
#	test to see if the value is within the range
#####
        if [ "${TYPE}" = "INT"  ]
        then
DONE=`echo ${VAL} "${MIN}" "${MAX}" | awk '{DONE="NO";if($1 >= $2) if($1 <=$3)DONE="YES"; printf DONE }'`
		if [ $DONE = "NO" ] 
		then
			while test $left != 0
			do

			$DIALOG --sleep 1 \
        			--title "Data Error" \
        			--infobox "The input value=${RETURNVALUE} \nis not in the range \n     $MIN to $MAX \nReenter.  You have $left $unit to read this..." 10 52
			left=`expr $left - 1`
			test $left = 1 && unit="second"
			done

		fi
        elif [ "${TYPE}" = "FLOAT"  ]
        then
DONE=`echo ${VAL} "${MIN}" "${MAX}" | awk '{DONE="NO";if($1 >= $2) if($1 <=$3)DONE="YES"; printf DONE }'`
		if [ $DONE = "NO" ] 
		then
			while test $left != 0
			do

			$DIALOG --sleep 1 \
        			--title "Data Error" \
        			--infobox "The input value=${RETURNVALUE} \nis not in the range \n     $MIN to $MAX \nReenter.  You have $left $unit to read this..." 10 52
			left=`expr $left - 1`
			test $left = 1 && unit="second"
			done

		fi
        fi
	;;
  1)
    echo "Cancel pressed.";;
  255)
    if test -s $tempfile ; then
      cat $tempfile
    else
      echo "ESC pressed."
    fi
    ;;
esac
RETURNVALUE=${VAL}
done
}


function getregion  {
$DIALOG --clear --title "Region" \
	--default-item "${REGION}" \
        --menu "Select region:(Region ANSS_code Vel_Model)" 20 72 10  \
"PK"   "Pakistan         ous   WUS (to 69 km deep)" \
	2> $tempfile
retval=$?
RETURNVALUE=`cat $tempfile`
case $retval in
  0)
    echo "Region: $REGION";;
  1)
    echo "Cancel pressed.";;
  255)
    echo "ESC pressed.";;
esac
}

function getscript  {
$DIALOG --clear --title "Region" \
	--default-item "${SCRIPT}" \
        --menu "Select processing script:" 20 71 7  \
"IRIS" "Use IRIS/NRCAN rdseed" \
	2> $tempfile
retval=$?
VALUE=`cat $tempfile`
case ${VALUE} in
	IRIS) RETURNVALUE="DOIRISRFTN" ;;
esac
case $retval in
  0)
    echo "Script: $SCRIPT";;
  1)
    echo "Cancel pressed.";;
  255)
    echo "ESC pressed.";;
esac
}

function getdecon  {
$DIALOG --clear --title "Region" \
	--default-item "${DECON}" \
        --menu "Select processing script:" 20 71 7  \
"DECONPZ" "Use polezero response" \
"DECONRESP" "Use RESP file response" \
"DECONNONE" "Do not remove response" \
	2> $tempfile
retval=$?
VALUE=`cat $tempfile`
case ${VALUE} in
	DECONPZ) RETURNVALUE="DECONPZ" ;;
	DECONRESP) RETURNVALUE="DECONRESP" ;;
	DECONNONE) RETURNVALUE="DECONNONE" ;;
esac
case $retval in
  0)
    echo "Script: $DECON";;
  1)
    echo "Cancel pressed.";;
  255)
    echo "ESC pressed.";;
esac
}

function mainmenu  {

DEFAULT_ITEM="Y"
choice=""
while [ "$choice" != "Q" ]
do
	CheckDate "${YEAR}" "${MONTH}" "${DAY}"

#####
#	initialize
#####
$DIALOG --clear --title "Main" \
	--default-item "${DEFAULT_ITEM}" \
        --menu "Select to change parameter (Tab for section, arrow keys to move, enter for selection" 25 71 17 \
	"Y" "YEAR          : ${YEAR}" \
	"m" "MONTH         : ${MONTH}" \
	"d" "DAY           : ${DAY}" \
	"H" "HOUR          : ${HOUR}" \
	"M" "MINUTE        : ${MINUTE}" \
	"S" "SECOND        : ${SECOND}" \
	"p" "MSEC          : ${MSEC}" \
	"L" "LATITUDE      : ${LATITUDE}" \
	"l" "LONGITUDE     : ${LONGITUDE}" \
	"h" "DEPTH         : ${DEPTH}" \
	"s" "MAGNITUDE     : ${MAG}" \
	"e" "NEIC EVID     : ${EVID}" \
	"f" "FELTID        : ${FEVID}" \
	"r" "State/Country : ${STATE}" \
	"P" "SCRIPT        : ${SCRIPT}" \
	"I" "DECON_TYPE    : ${DECON}" \
	"Q" "Quit" \
	2> $tempfile

retval=$?
choice=`cat $tempfile`
case $retval in
  0)
	case $choice in
	Y) 
		DEFAULT_ITEM="m" 
		TITLE="YEAR"
		TYPE="INT"
		FORMAT="%4.4d"
		TEXT="Enter YEAR (e.g.,2008)"
		MINVAL="1900"
		MAXVAL="2100"
		getreturnvalue "${YEAR}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		YEAR="${RETURNVALUE}"
		;;
	m) 
		DEFAULT_ITEM="d" 
		TITLE="MONTH"
		TYPE="INT"
		FORMAT="%2.2d"
		TEXT="Enter MONTH (e.g.,02)"
		MINVAL="01"
		MAXVAL="12"
		getreturnvalue "${MONTH}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		MONTH="${RETURNVALUE}"
		;;
	d) 
		DEFAULT_ITEM="H" 
		TITLE="DAY"
		TYPE="INT"
		FORMAT="%2.2d"
		TEXT="Enter DAY (e.g.,02)"
		MINVAL="01"
		MAXVAL="31"
		getreturnvalue "${DAY}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		DAY="${RETURNVALUE}"
		;;
	H) 
		DEFAULT_ITEM="M" 
		TITLE="HOUR"
		TYPE="INT"
		FORMAT="%2.2d"
		TEXT="Enter HOUR (e.g.,02)"
		MINVAL="00"
		MAXVAL="23"
		getreturnvalue "${HOUR}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		HOUR="${RETURNVALUE}"
		;;
	M) 
		DEFAULT_ITEM="S" 
		TITLE="MINUTE"
		TYPE="INT"
		FORMAT="%2.2d"
		TEXT="Enter MINUTE (e.g.,02)"
		MINVAL="00"
		MAXVAL="59"
		getreturnvalue "${MINUTE}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		MINUTE="${RETURNVALUE}"
		;;
	S) 
		DEFAULT_ITEM="p" 
		TITLE="SECOND"
		TYPE="INT"
		FORMAT="%2.2d"
		TEXT="Enter SECOND (e.g.,02)"
		MINVAL="00"
		MAXVAL="59"
		getreturnvalue "${SECOND}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		SECOND="${RETURNVALUE}"
		;;
	p) 
		DEFAULT_ITEM="L" 
		TITLE="MSEC"
		TYPE="INT"
		FORMAT="%3.3d"
		TEXT="Enter MILLISECOND (e.g.,002)"
		MINVAL="000"
		MAXVAL="999"
		getreturnvalue "${MSEC}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		MSEC="${RETURNVALUE}"
		;;
	L) 
		DEFAULT_ITEM="l" 
		TITLE="LATITUDE"
		TYPE="FLOAT"
                FORMAT="%9.4f"
		TEXT="Enter LATITUDE (+ is N, - is S)"
		MINVAL="-90.0"
		MAXVAL="90.0"
		getreturnvalue "${LATITUDE}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		LATITUDE="${RETURNVALUE}"
		;;
	l) 
		DEFAULT_ITEM="h" 
		TITLE="LONGITUDE"
		TYPE="FLOAT"
                FORMAT="%10.4f"
		TEXT="Enter LONGITUDE (+ is E, - is W)"
		MINVAL="-180.0"
		MAXVAL="180.0"
		getreturnvalue "${LONGITUDE}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		LONGITUDE="${RETURNVALUE}"
		;;
	h) 
		DEFAULT_ITEM="s" 
		TITLE="DEPTH"
		TYPE="FLOAT"
		FORMAT="%5.1f"
		TEXT="Enter DEPTH in km"
		MINVAL="0.0"
		MAXVAL="999.9"
		getreturnvalue "${DEPTH}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		DEPTH="${RETURNVALUE}"
		;;
	s) 
		DEFAULT_ITEM="e" 
		TITLE="MAGNITUDE"
		TYPE="FLOAT"
		FORMAT="%5.2f"
		TEXT="Enter Magnitude"
		MINVAL="-3.0"
		MAXVAL="10.0"
		getreturnvalue "${MAGNITUDE}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		MAG="${RETURNVALUE}"
		;;
	e) 
		DEFAULT_ITEM="f" 
		TITLE="NEIC EVID"
		TYPE="STRING"
		FORMAT="%s"
		TEXT="Enter NEIC EVID or NONE"
		getreturnvalue "${EVID}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		EVID="${RETURNVALUE}"
		;;
	f) 
		DEFAULT_ITEM="r" 
		TITLE="FEVID"
		TYPE="STRING"
		FORMAT="%s"
		TEXT="Enter DidYouFeelid FeltID"
		getreturnvalue "${FEVID}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		FEVID="${RETURNVALUE}"
		;;
	r) 
		DEFAULT_ITEM="P" 
		TITLE="STATE"
		TYPE="STRING"
		FORMAT="%s"
		TEXT="Enter State/Province/Country name"
		getreturnvalue "${STATE}" "${MINVAL}" "${MAXVAL}" "${FORMAT}" "${TYPE}" "${TEXT}" "${TITLE}"
		STATE="${RETURNVALUE}"
		;;
	P) 
		DEFAULT_ITEM="I" 
		TITLE="SCRIPT"
		TYPE="STRING"
		FORMAT="%s"
		TEXT="Enter Processing Script:  DOIRISRFTN"
		getscript
		SCRIPT="${RETURNVALUE}"
		;;
	I) 
		DEFAULT_ITEM="Q" 
		TITLE="DECON_TYPE"
		TYPE="STRING"
		FORMAT="%s"
		TEXT="Enter Processing Script:  DECONNONE DECONPZ DECONRESP"
		getdecon
		DECON="${RETURNVALUE}"
		;;
	esac ;;
  1)
    echo "Cancel pressed.";;
  255)
    echo "ESC pressed.";;
esac
done
}

clear

#####
#	main processing 
#####
mainmenu
rm -f DO
clear
cat > DO << EOF
#!/bin/sh 
#####
# valid regions
# REG    Region             FELTID  VELOCITY_MODEL
# PK     Pakistan            ous       WUS  (to 69 km deep)
#####
# Command syntax:
#DOCWBREG YEAR MO DY HR MN SC MSC  LAT     LON    DEP  MAG REG  NEIC FELTID STATE/COUNTRY DECON_TYPE
#####
EOF

echo ${SCRIPT} \"$YEAR\" \"$MONTH\" \"$DAY\" \"$HOUR\" \"$MINUTE\" \"$SECOND\" \"$MSEC\" \"$LATITUDE\" \"$LONGITUDE\" \"$DEPTH\" \"$MAG\" \"$REGION\" \"$EVID\" \"$FEVID\" \"$STATE\" \"$DECON\">> DO
chmod +x DO
cat DO

echo '************************************************************'
echo 'To enter the command DO to begin the moment tensor procedure'
echo '************************************************************'
